/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.samples.sm;

import org.wso2.registry.web.custom.CustomUIProcessor;
import org.wso2.registry.web.custom.CustomUIBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.Resource;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.HashMap;

public class ServerUIProcessor extends CustomUIProcessor {

    private static final String MEDIA_TYPE = "sv";

    private Map <String, String> createViews = new HashMap <String, String> ();
    private Map <String, String> resourceViews = new HashMap <String, String> ();

    private static final String BASIC_CREATE = "basic";
    private static final String DETAILS_CREATE = "details";

    private static final String DETAILS_VIEW = "details";
    private static final String BASIC_VIEW = "basic";
    private static final String EDIT_VIEW = "edit";

    // commands
    private static final String ADD_BASIC = "addBasic";
    private static final String ADD_DETAILS = "addDetails";
    private static final String EDIT_DETAILS = "editDetails";
    private static final String VERIFY_ADMIN = "verifyAdmin";

    private static final String VIEW1_JSP = "server-resource-view1.jsp";
    private static final String VIEW2_JSP = "server-resource-view2.jsp";
    private static final String CREATE1_JSP = "/admin/server-create-view1.jsp";
    private static final String CREATE2_JSP = "/admin/server-create-view2.jsp";

    public ServerUIProcessor() {
        createViews.put(BASIC_CREATE, "Basic");
        createViews.put(DETAILS_CREATE, "Details");

        resourceViews.put(DETAILS_VIEW, "Details");
        resourceViews.put(BASIC_VIEW, "View basic information");
        resourceViews.put(EDIT_VIEW, "Edit server information");
    }

    public String getDefaultCreateView() {
        return BASIC_CREATE;
    }

    public String getDefaultResourceView() {
        return BASIC_VIEW;
    }

    public Map<String, String> getCreateViews() {
        return createViews;
    }

    public Map<String, String> getResourceViews() {
        return resourceViews;
    }

    public CustomUIBean process(String command, HttpServletRequest request)
            throws RegistryException {

        if (ADD_BASIC.equals(command)) {
            
            String name = request.getParameter("name");
            String desc = request.getParameter("desc");
            String parentPath = request.getParameter(UIConstants.PARENT_PATH);
            String resourceName = request.getParameter(UIConstants.RESOURCE_NAME);

            String path = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
            String content = "name:" + name + "\ndescription:" + desc;

            Resource resource = registry.newResource();
            resource.setMediaType(MEDIA_TYPE);
            resource.setContent(content);
            registry.put(path, resource);

            CustomUIBean cuib = new CustomUIBean();
            cuib.setResponseMethod(CustomUIBean.REDIRECT);
            cuib.setResourcePath(parentPath);
            return cuib;

        } else if (ADD_DETAILS.equals(command)) {

            String name = request.getParameter("name");
            String desc = request.getParameter("desc");
            String ip = request.getParameter("ip");
            String port = request.getParameter("port");
            String admin = request.getParameter("admin");
            String parentPath = request.getParameter(UIConstants.PARENT_PATH);
            String resourceName = request.getParameter(UIConstants.RESOURCE_NAME);

            String path = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
            String content = "name:" + name +
                    "\ndescription:" + desc +
                    "\nip:" + ip +
                    "\nports:" + port +
                    "\nadmin:" + admin;

            Resource resource = registry.newResource();
            resource.setMediaType(MEDIA_TYPE);
            resource.setContent(content);
            registry.put(path, resource);

            CustomUIBean cuib = new CustomUIBean();
            cuib.setResponseMethod(CustomUIBean.REDIRECT);
            cuib.setResourcePath(parentPath);
            return cuib;

        } else if (EDIT_DETAILS.equals(command)) {

            String name = request.getParameter("name");
            String desc = request.getParameter("desc");
            String ip = request.getParameter("ip");
            String port = request.getParameter("port");
            String admin = request.getParameter("admin");
            String resourcePath = request.getParameter(UIConstants.RESOURCE_PATH);

            String content = "name:" + name +
                    "\ndescription:" + desc +
                    "\nip:" + ip +
                    "\nports:" + port +
                    "\nadmin:" + admin;

            Resource resource = registry.get(resourcePath);
            resource.setContent(content);
            registry.put(resourcePath, resource);

            CustomUIBean cuib = new CustomUIBean();
            cuib.setResponseMethod(CustomUIBean.REDIRECT);
            cuib.setResourcePath(resourcePath);
            return cuib;
            
        } else if (VERIFY_ADMIN.equals(command)) {

            String admin = request.getParameter("admin");

            CustomUIBean cuib = new CustomUIBean();
            cuib.setResponseMethod(CustomUIBean.CONTENT_ONLY);
            if ("chamith".equals(admin)) {
                cuib.setContentString("Admin available");
            } else {
                cuib.setContentString("Admin not available");
            }
            return cuib;

        } else {
            String msg = "Unsupported command: " + command + " for the media type: " + MEDIA_TYPE;
            throw new RegistryException(msg);
        }
    }

    public CustomUIBean processCreateView(HttpServletRequest request, String viewKey)
            throws RegistryException {

        if (BASIC_CREATE.equals(viewKey)) {
            CustomUIBean cuib = new CustomUIBean();
            cuib.setParentPath(request.getParameter(UIConstants.PARENT_PATH));
            cuib.setJspPath(CREATE1_JSP);
            return cuib;

        } else if (DETAILS_CREATE.equals(viewKey)) {
            CustomUIBean cuib = new CustomUIBean();
            cuib.setParentPath(request.getParameter(UIConstants.PARENT_PATH));
            cuib.setJspPath(CREATE2_JSP);
            return cuib;

        } else {
            String msg = "Unsupported create view: " + viewKey;
            throw new RegistryException(msg);
        }
    }

    public CustomUIBean processResourceView(
            String path, String viewKey, HttpServletRequest request) throws RegistryException {

        if (DETAILS_VIEW.equals(viewKey)) {
            return viewDetails(path);

        } else if (BASIC_VIEW.equals(viewKey)) {
            return viewBasics(path);

        } else if (EDIT_VIEW.equals(viewKey)) {
            return viewDetailsEdit(path);

        } else {
            String msg = "Unsupported view: " + viewKey;
            throw new RegistryException(msg);
        }
    }

    private CustomUIBean viewDetails(String path) throws RegistryException {

        Resource resource = registry.get(path);
        byte[] contentBytes = (byte[]) resource.getContent();
        resource.discard();

        String content = new String(contentBytes);
        String[] fields = content.split("\n");

        Map <String, String> data = new HashMap <String, String> ();
        for (int i = 0; i < fields.length; i++) {
            String field = fields[i];
            String[] parts = field.split(":");
            data.put(parts[0], parts[1]);
        }

        ServerUIBean bean = new ServerUIBean();
        bean.setName(data.get("name"));
        bean.setDescription(data.get("description"));
        bean.setIp(data.get("ip"));
        bean.setPort(data.get("ports"));
        bean.setAdmin(data.get("admin"));

        bean.setJspPath(VIEW1_JSP);
        return bean;
    }

    private CustomUIBean viewBasics(String path) throws RegistryException {

        Resource resource = registry.get(path);
        byte[] contentBytes = (byte[]) resource.getContent();
        resource.discard();

        String content = new String(contentBytes);
        String[] fields = content.split("\n");

        Map <String, String> data = new HashMap <String, String> ();
        for (int i = 0; i < fields.length; i++) {
            String field = fields[i];
            String[] parts = field.split(":");
            data.put(parts[0], parts[1]);
        }

        CustomUIBean bean = new ServerUIBean();
        String name = data.get("name");
        String ip = data.get("ip");
        String htmlContent = "<h2>Basic Information</h2>Name: " +
                name + "<br/>IP: " + ip + "<br/><br/>";
        bean.setContentString(htmlContent);

        return bean;
    }

    private CustomUIBean viewDetailsEdit(String path) throws RegistryException {

        Resource resource = registry.get(path);
        byte[] contentBytes = (byte[]) resource.getContent();
        resource.discard();

        String content = new String(contentBytes);
        String[] fields = content.split("\n");

        Map <String, String> data = new HashMap <String, String> ();
        for (int i = 0; i < fields.length; i++) {
            String field = fields[i];
            String[] parts = field.split(":");
            data.put(parts[0], parts[1]);
        }

        ServerUIBean bean = new ServerUIBean();
        bean.setResourcePath(path);

        bean.setName(data.get("name"));
        bean.setDescription(data.get("description"));
        bean.setIp(data.get("ip"));
        bean.setPort(data.get("ports"));
        bean.setAdmin(data.get("admin"));

        bean.setJspPath(VIEW2_JSP);
        return bean;
    }
}
